/*
 * $Id$
 * Copyright (c) 2001 Stefan Matthias Aust.  All Rights Reserved.
 *
 * This file is part of the 3plus4 expression parser.  You are granted the
 * right to use this code in GPL based projects in which case this code is
 * also protected by the GPL, or in other projects as long as you give credit
 * to Stefan Matthias Aust (sma@3plus4.de) and make all modifications or
 * extensions to this code freely available.
 */
package de.bo.expressionparser;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * This class implements a simple function plotter.
 *
 * @author Stefan Matthias Aust <sma@3plus4.de>
 */
public class Plotter extends JFrame {

    /**
     * Constructs a new function plotter.
     */
    public Plotter() {
        super("BO Expression Parser");
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });
        final JTextField t = new JTextField("sin x");
        final JComponent c = new JComponent() {
            public void paintComponent(Graphics g) {
                plot(this, g);
            }
        };
        t.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                parse(t.getText());
                c.repaint();
            }
        });
        JLabel l = new JLabel("v3 - Copyright (c) 2001 Stefan Matthias Aust", JLabel.RIGHT);
        l.setFont(l.getFont().deriveFont(9f));
        getContentPane().add(t, BorderLayout.NORTH);
        getContentPane().add(c, BorderLayout.CENTER);
        getContentPane().add(l, BorderLayout.SOUTH);
        setSize(400, 400);
    }

    /**
     * Converts the specified string into a hiearchy of function nodes which
     * can then used to compute <code>f(x)</code> values.
     * @param s the expression to parse
     */
    private void parse(String s) {
        try {
            function = Parser.parse(s);
        } catch (Exception e) {
            function = null;
        }
    }

    /**
     * Plots the parsed function or shows a help text.
     * @param c the component to draw onto
     * @param g the component's <code>Graphics</code> to use
     */
    private void plot(JComponent c, Graphics g) {
        int width = c.getWidth();
        int height = c.getHeight();

        g.setColor(c.getBackground());
        g.fillRect(0, 0, width, height);
        width /= 2;
        height /= 2;

        if (function == null) {
            String s = "Enter a function and press RETURN";
            g.setFont(c.getFont());
            g.setColor(c.getForeground());
            g.drawString(s, width - g.getFontMetrics().stringWidth(s) / 2, height);
            return;
        }

        g.setColor(c.getBackground().darker());
        g.drawLine(0, height, width * 2, height);
        g.drawLine(width, 0, width, height * 2);
        g.setColor(c.getForeground());

        float x1, y1, x2, y2, s = 20;
        x1 = x2 = -8f;
        y1 = y2 = (float)function.eval(x2);
        do {
            x1 += 0.1f;
            y1 = (float)function.eval(x1);
            g.drawLine(Math.round(x1 * s) + width, Math.round(height - y1 * s),
                       Math.round(x2 * s) + width, Math.round(height - y2 * s));
            y2 = y1;
            x2 = x1;
        } while (x2 <= 8f);
    }

    /** The function's hierarchical object node representation */
    private Node function;

    static public void main(String[] args) {
        new Plotter().show();
    }
}