/*
 * $Id$
 * Copyright (c) 2001 Stefan Matthias Aust.  All Rights Reserved.
 *
 * This file is part of the 3plus4 expression parser.  You are granted the
 * right to use this code in GPL based projects in which case this code is
 * also protected by the GPL, or in other projects as long as you give credit
 * to Stefan Matthias Aust (sma@3plus4.de) and make all modifications or
 * extensions to this code freely available.
 */
package de.bo.expressionparser;

/**
 * A Token represents one piece of the input stream.
 * @see Scanner
 *
 * @author Stefan Matthias Aust <sma@3plus4.de>
 */
class Token {
    public static final Token EOF = new Token();
    public static final Token VARIABLE = new Token('x');
    public static final Token PLUS = new Token('+');
    public static final Token MINUS = new Token('-');
    public static final Token TIMES = new Token('*');
    public static final Token DIVIDE = new Token('/');
    public static final Token POWER = new Token('^');
    public static final Token LPAREN = new Token('(');
    public static final Token RPAREN = new Token(')');

    public Token() {
        code = -1;
    }

    public Token(int ch) {
        code = ch;
    }

    public Token(String name) {
        code = 'n';
        object = name;
    }

    public Token(Double number) {
        code = '0';
        object = number;
    }

    public String getName() {
        return (String)object;
    }

    public double getNumber() {
        return ((Double)object).doubleValue();
    }

    public boolean isName() {
        return code == 'n';
    }

    public boolean isNumber() {
        return code == '0';
    }

    private int code;
    private Object object;
}